@@ -36,6 +36,10 @@
 
 // This is a C++ file in order to use the C++ BeOS API
 
+#include "cairoint.h"
+
+#include "cairo-beos.h"
+
 #include <new>
 
 #include <Bitmap.h>
@@ -47,9 +51,6 @@
 #include <Window.h>
 #include <Locker.h>
 
-#include "cairoint.h"
-#include "cairo-beos.h"
-
 #define CAIRO_INT_STATUS_SUCCESS (cairo_int_status_t)(CAIRO_STATUS_SUCCESS)
 
 struct cairo_beos_surface_t {
@@ -976,33 +977,3 @@ cairo_beos_surface_create_for_bitmap (BView*   view,
 {
     return _cairo_beos_surface_create_internal(view, bmp);
 }
-
-// ---------------------------------------------------------------------------
-// Cairo uses locks without explicit initialization. To support that, we
-// provide a class here which manages the locks and is in global scope, so the
-// compiler will instantiate it on library load and destroy it on library
-// unload.
-
-class BeLocks {
-    public:
-	BLocker cairo_toy_font_face_hash_table_mutex;
-	BLocker cairo_scaled_font_map_mutex;
-	BLocker cairo_ft_unscaled_font_map_mutex;
-};
-
-static BeLocks locks;
-
-void* cairo_toy_font_face_hash_table_mutex = &locks.cairo_toy_font_face_hash_table_mutex;
-void* cairo_scaled_font_map_mutex = &locks.cairo_scaled_font_map_mutex;
-void* cairo_ft_unscaled_font_map_mutex = &locks.cairo_ft_unscaled_font_map_mutex;
-
-void _cairo_beos_lock (void* locker) {
-    BLocker* bLocker = reinterpret_cast<BLocker*>(locker);
-    bLocker->Lock();
-}
-
-void _cairo_beos_unlock (void* locker) {
-    BLocker* bLocker = reinterpret_cast<BLocker*>(locker);
-    bLocker->Unlock();
-}
-
